import 'package:agora_rtc_engine/agora_rtc_engine.dart';
import 'package:agora_rtc_wrapper/agora_rtc_wrapper.dart';
import 'package:flutter/material.dart';

/// Created by taohid on 15,May, 2020
/// Email: taohid32@gmail.com

class AgoraRtcWrapperImpl implements AgoraRtcWrapper {
  final _users = <int>[];
  bool _muted = false;

  AgoraRtcWrapperImpl(
    String appId, {
    OnError onError,
    OnJoinSuccess onJoinSuccess,
    VoidCallback onLeaveChannel,
    OnUserJoined onUserJoined,
    OnUserOffline onUserOffline,
    OnFirstRemoteVideoFrame onFirstRemoteVideoFrame,
  }) {
    _initAgoraEngine(appId);
    _setEventListener(
      onError,
      onJoinSuccess,
      onLeaveChannel,
      onUserJoined,
      onUserOffline,
      onFirstRemoteVideoFrame,
    );
  }

  Future _initAgoraEngine(String appId) async {
    AgoraRtcEngine.create(appId);
    AgoraRtcEngine.enableVideo();
    AgoraRtcEngine.enableAudio();
    AgoraRtcEngine.setParameters(
        '{\"che.video.lowBitRateStreamParameter\":{\"width\":320,\"height\":180,\"frameRate\":15,\"bitRate\":140}}');
    AgoraRtcEngine.setChannelProfile(ChannelProfile.Communication);
    VideoEncoderConfiguration config = VideoEncoderConfiguration();
    config.orientationMode = VideoOutputOrientationMode.FixedPortrait;
    AgoraRtcEngine.setVideoEncoderConfiguration(config);
  }

  void _setEventListener(
    OnError onError,
    OnJoinSuccess onJoinSuccess,
    VoidCallback onLeaveChannel,
    OnUserJoined onUserJoined,
    OnUserOffline onUserOffline,
    OnFirstRemoteVideoFrame onFirstRemoteVideoFrame,
  ) {
    AgoraRtcEngine.onError = onError;
    AgoraRtcEngine.onJoinChannelSuccess = onJoinSuccess;
    AgoraRtcEngine.onFirstRemoteVideoFrame = onFirstRemoteVideoFrame;

    AgoraRtcEngine.onUserJoined = (int uid, int elapsed) {
      _users.add(uid);
      onUserJoined != null
          ? onUserJoined(uid, elapsed)
          : print('User Joined Listener Not Set');
    };

    AgoraRtcEngine.onLeaveChannel = () {
      _users.clear();
      onLeaveChannel != null
          ? onLeaveChannel()
          : print('Leave Listener Not Set');
    };

    AgoraRtcEngine.onUserOffline = (int uid, int reason) {
      _users.remove(uid);
      onUserOffline != null
          ? onUserOffline(uid, reason)
          : print('User Offline Listener Not Set');
    };
  }

  @override
  List<int> getUsers() {
    return _users;
  }

  @override
  void joinChannel(String channelName) async {
    AgoraRtcEngine.joinChannel(null, channelName, null, 0);
  }

  @override
  void enableLog() {
    AgoraRtcEngine.setParameters("{\"rtc.log_filter\": 65535}");
  }

  /// for self view uid is always 0
  @override
  Widget get getSelfVideoView =>
      AgoraRenderWidget(0, local: true, preview: true);

  @override
  Widget getVideoView(int uid) => AgoraRenderWidget(uid);

  @override
  void muteAudio(bool mute) {
    AgoraRtcEngine.muteLocalAudioStream(mute);
    _muted = mute;
  }

  @override
  void switchCamera() {
    AgoraRtcEngine.switchCamera();
  }

  @override
  Widget getDefaultController({
    @required VoidCallback onCallEnd,
    @required ValueChanged<bool> onMute,
    Color selectedButtonColor = Colors.blueAccent,
  }) {
    return Container(
      alignment: Alignment.bottomCenter,
      padding: const EdgeInsets.symmetric(vertical: 48),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          RawMaterialButton(
            onPressed: () => onMute(_muted),
            child: Icon(
              _muted ? Icons.mic_off : Icons.mic,
              color: _muted ? Colors.white : selectedButtonColor,
              size: 20.0,
            ),
            shape: CircleBorder(),
            elevation: 2.0,
            fillColor: _muted ? selectedButtonColor : Colors.white,
            padding: const EdgeInsets.all(12.0),
          ),
          RawMaterialButton(
            onPressed: onCallEnd,
            child: Icon(
              Icons.call_end,
              color: Colors.white,
              size: 35.0,
            ),
            shape: CircleBorder(),
            elevation: 2.0,
            fillColor: Colors.redAccent,
            padding: const EdgeInsets.all(15.0),
          ),
          RawMaterialButton(
            onPressed: switchCamera,
            child: Icon(
              Icons.switch_camera,
              color: selectedButtonColor,
              size: 20.0,
            ),
            shape: CircleBorder(),
            elevation: 2.0,
            fillColor: Colors.white,
            padding: const EdgeInsets.all(12.0),
          )
        ],
      ),
    );
  }

  @override
  void dispose() {
    _users.clear();
    AgoraRtcEngine.leaveChannel();
    AgoraRtcEngine.destroy();
  }
}
